#include "CircleNode.h"

#include "FontsManager.h"
#include "SceneModel.h"
#include "AbstractMesh.h"
#include "MeshPrimitives.h"
#include "SceneInfo.h"

#include <iostream>

CircleNode::CircleNode(const Point& location, double radius)
:    GeometryNode(location),
     mRadius(radius)
{

}

CircleNode::Type CircleNode::getType()
{
    return Type::Circle;
}

double CircleNode::getRadius() const
{
    return mRadius;
}

SceneItem* CircleNode::getSceneItem(std::size_t idx) const
{
    if (idx == 0)
    {
        return mBackgroundItem.get();
    }
    else
    {
        return nullptr;
    }
}

std::size_t CircleNode::getNumSceneItems() const
{
    return 1;
}

void CircleNode::setRadius(double radius)
{
    if (mRadius != radius)
    {
        mRadius = radius;
        mTransformIsDirty = true;
    }
}

void CircleNode::update(SceneInfo* sceneInfo)
{
    if (!mBackgroundItem || mGeometryIsDirty)
    {
        auto mesh = MeshPrimitives::buildCircleAsTriMesh();

        if (!mBackgroundItem)
        {
            mBackgroundItem = std::make_unique<SceneModel>(std::move(mesh));
            mBackgroundItem->setName(mName + "_Model");
            mBackgroundItem->setFillColor(mFillColor);
        }
        else
        {
            mBackgroundItem->updateMesh(std::move(mesh));
        }
        mGeometryIsDirty = false;
    }

    if (mTransformIsDirty)
    {
        mBackgroundItem->updateTransform({mLocation, 2*mRadius, 2*mRadius});
        mTransformIsDirty = false;
    }

    if (mMaterialIsDirty)
    {
        mBackgroundItem->setFillColor(mFillColor);
        mMaterialIsDirty = false;
    }
}
